@@ -106,7 +106,9 @@ struct {
   int gtpversion;
   struct ul255_t pco;
   struct ul255_t qos;
+  uint16_t cch;
   struct ul255_t apn;
+  uint8_t selmode;
   struct ul16_t msisdn;
 } options;
 
@@ -224,6 +226,7 @@ int process_options(int argc, char **argv) {
     printf("debug: %d\n", args_info.debug_flag);
     if (args_info.imsi_arg) printf("imsi: %s\n", args_info.imsi_arg);
     printf("qos: %#08x\n", args_info.qos_arg);
+    printf("charging: %#04x\n", args_info.charging_arg);
     if (args_info.apn_arg) printf("apn: %s\n", args_info.apn_arg);
     if (args_info.msisdn_arg) printf("msisdn: %s\n", args_info.msisdn_arg);
     if (args_info.uid_arg) printf("uid: %s\n", args_info.uid_arg);
@@ -257,6 +260,7 @@ int process_options(int argc, char **argv) {
       printf("debug: %d\n", args_info.debug_flag);
       if (args_info.imsi_arg) printf("imsi: %s\n", args_info.imsi_arg);
       printf("qos: %#08x\n", args_info.qos_arg);
+      printf("charging: %#04x\n", args_info.charging_arg);
       if (args_info.apn_arg) printf("apn: %s\n", args_info.apn_arg);
       if (args_info.msisdn_arg) printf("msisdn: %s\n", args_info.msisdn_arg);
       if (args_info.uid_arg) printf("uid: %s\n", args_info.uid_arg);
@@ -411,6 +415,9 @@ int process_options(int argc, char **argv) {
   options.qos.v[2] = (args_info.qos_arg) & 0xff;
   options.qos.v[1] = ((args_info.qos_arg) >> 8) & 0xff;
   options.qos.v[0] = ((args_info.qos_arg) >> 16) & 0xff;
+
+  /* charging                                                        */
+  options.cch = args_info.charging_arg;
   
   /* contexts                                                        */
   if (args_info.contexts_arg > MAXCONTEXTS) {
@@ -441,6 +448,12 @@ int process_options(int argc, char **argv) {
   options.apn.v[0] = (char) strlen(args_info.apn_arg);
   strncpy(&options.apn.v[1], args_info.apn_arg, sizeof(options.apn.v)-1);
   printf("Using APN:             %s\n", args_info.apn_arg);
+
+
+  /* selmode */
+  options.selmode = args_info.selmode_arg;
+  printf("Using selection mode:  %d\n", args_info.selmode_arg);
+
   
   /* msisdn                                                          */
   if (strlen(args_info.msisdn_arg)>(sizeof(options.msisdn.v)-1)) {
@@ -473,7 +486,7 @@ int process_options(int argc, char **argv) {
   options.pco.v[0] = 0x80; /* PPP */
   options.pco.v[1] = 0xc0; /* PAP */
   options.pco.v[2] = 0x23; 
-  options.pco.v[3] = 0x12; /* Length of protocol contents */
+  options.pco.v[3] = strlen(args_info.uid_arg) + strlen(args_info.pwd_arg) + 6;
   options.pco.v[4] = 0x01; /* Authenticate request */
   options.pco.v[5] = 0x01;
   options.pco.v[6] = 0x00; /* MSB of length */
@@ -1128,7 +1141,7 @@ int main(int argc, char **argv)
     pdp->qos_req.v[0] = 0x00;
     memcpy(pdp->qos_req.v+1, options.qos.v, options.qos.l);
     
-    pdp->selmode = 0x01; /* MS provided APN, subscription not verified */
+    pdp->selmode = options.selmode;
     
     if (options.apn.l > sizeof(pdp->apn_use.v)) {
       sys_err(LOG_ERR, __FILE__, __LINE__, 0, "APN length too big");
@@ -1168,6 +1181,9 @@ int main(int argc, char **argv)
     pdp->hisaddr0 = options.remote;
     pdp->hisaddr1 = options.remote;
 
+    pdp->cch_pdp = options.cch; /* 2048 = Normal, 1024 = Prepaid, 
+				   512 = Flat rate, 256 = Hot billing */
+
     /* Create context */
     /* We send this of once. Retransmissions are handled by gtplib */
     gtp_create_context_req(gsn, pdp, &iparr[n]);
